#!/usr/bin/perl
#################################################################################
#										#
# table generator for libmio							#
# copyright (c) 2006 Joerg Wolfram (joerg@jcwolfram.de)				#
#										#
#										#
# This program is free software; you can redistribute it and/or			#
# modify it under the terms of the GNU General Public License			#
# as published by the Free Software Foundation; either version 2		#
# of the License, or (at your option) any later version.			#
#										#
# This program is distributed in the hope that it will be useful,		#
# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU		#
# General Public License for more details.					#
#										#
# You should have received a copy of the GNU General Public			#
# License along with this library; if not, write to the				#
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
# Boston, MA 02111-1307, USA.							#
#										#
#################################################################################

$takt=16000000;
$pwm=256;
$maxcount=65536;
$fakt=3.5;

$count=0;
$ncount=1;
$cmode=0;
$cline=0;
$cpos=0;

sub clear_cells
{
    for($i=0;$i<7680;$i++)
    {
	$cell[$i]=00;
    }
    for($i=0;$i<2048;$i++)
    {
	$mcell[$i]=0;
    }
}

sub get_pixel
{
    $nb=0;
    if ($z =~ /^\./) {$nb=0}
    elsif ($z =~ /^b/) {$nb=2}
    elsif ($z =~ /^r/) {$nb=4}
    elsif ($z =~ /^m/) {$nb=6}
    elsif ($z =~ /^g/) {$nb=8}
    elsif ($z =~ /^c/) {$nb=10}
    elsif ($z =~ /^y/) {$nb=12}
    elsif ($z =~ /^w/) {$nb=12}
    elsif ($z =~ /^B/) {$nb=2}
    elsif ($z =~ /^R/) {$nb=4}
    elsif ($z =~ /^M/) {$nb=6}
    elsif ($z =~ /^G/) {$nb=8}
    elsif ($z =~ /^C/) {$nb=10}
    elsif ($z =~ /^Y/) {$nb=12}
    elsif ($z =~ /^W/) {$nb=14}
    $z=$';
    return $nb;
}

sub get_nibble
{
    $nb=255;
    if ($z =~ /^0/) {$nb=0}
    elsif ($z =~ /^1/) {$nb=1}
    elsif ($z =~ /^2/) {$nb=2}
    elsif ($z =~ /^3/) {$nb=3}
    elsif ($z =~ /^4/) {$nb=4}
    elsif ($z =~ /^5/) {$nb=5}
    elsif ($z =~ /^6/) {$nb=6}
    elsif ($z =~ /^7/) {$nb=7}
    elsif ($z =~ /^8/) {$nb=8}
    elsif ($z =~ /^9/) {$nb=9}
    elsif ($z =~ /^a/) {$nb=10}
    elsif ($z =~ /^b/) {$nb=11}
    elsif ($z =~ /^c/) {$nb=12}
    elsif ($z =~ /^d/) {$nb=13}
    elsif ($z =~ /^e/) {$nb=14}
    elsif ($z =~ /^f/) {$nb=15}
    elsif ($z =~ /^,/) {$nb=100}
    $z=$';
    return $nb;
}

&clear_cells;

###############################################################################
# read character cells
###############################################################################
open (RTAB, "ctab_256C8.dat");
while(<RTAB>)
{
    $z=$_;
    chomp($z);
    if ($z =~ /cell/) 
    {
	$cline=0;
    }
    
    elsif (length($z)>5)
    {
	$pix1=&get_pixel();
	$pix2=&get_pixel();
	$pix3=&get_pixel();
	$pix4=&get_pixel();
	$pix5=&get_pixel();
	$pix6=&get_pixel();
	$mbyte=0;
	if ($pix2 > 0) 
	{
	    $mbyte+=128;
	}
	if ($pix3 > 0) 
	{
	    $mbyte+=64;
	}
	if ($pix4 > 0)
	{
	    $mbyte+=32;
	}
	if ($pix5 > 0)
	{
	    $mbyte+=16;
	}
	if ($pix6 > 0) 
	{
	    $mbyte+=8;
	}
	
	if (($cline>0) && ($cline <11))
	{
	    $cell[3*$count+768*($cline-1)]=$pix4+16*$pix1;
	    $cell[3*$count+768*($cline-1)+1]=$pix5+16*$pix2;
	    $cell[3*$count+768*($cline-1)+2]=$pix6+16*$pix3;
	    if ($count<128)
	    {
		$mcell[$count+256*($cline-1)]=$mbyte;
		$mcell[$count+256*($cline-1)+128]=255-$mbyte;
	    }	
	}    
	$cline++;
        if ($cline==12)
	{
	    $cline=0;
	    $count++;
	}
    }

}

close(RTAB);

###############################################################################
# write charecter table
###############################################################################
open (WTAB, ">../cchartable.inc");
    for($i=0;$i<480;$i++)
    {
        print WTAB "        .db "; 
        for($j=0;$j<16;$j++)
        {
	    $cellpos=$i*16+$j;
	    $y=$cell[$cellpos];
	    if ($y<1) {printf WTAB "0x00"} 
	    else {printf WTAB "%#2.2x",$y};
	    if ($j<15) {print WTAB ","}
	    else {print WTAB "\n"};
	}
    }
close(WTAB);

###############################################################################
# write editor text table
###############################################################################
open (WTAB, ">../mchartable.inc");
    for($i=0;$i<160;$i++)
    {
        print WTAB "        .db "; 
        for($j=0;$j<16;$j++)
        {
	    $cellpos=$i*16+$j;
	    $y=$mcell[$cellpos];
	    if ($y<1) {printf WTAB "0x00"} 
	    else {printf WTAB "%#2.2x",$y};
	    if ($j<15) {print WTAB ","}
	    else {print WTAB "\n"};
	}
    }
close(WTAB);

