;################################################################################
;#										#
;# chipbasic - single chip basic computer with ATmega32				#
;# line compressor/expander							#
;# copyright (c) 2007 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;#										#
;# This program is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public License			#
;# as published by the Free Software Foundation; either version 2		#
;# of the License, or (at your option) any later version.			#
;#										#
;# This program is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

;-----------------------------------------------------------------------
; compress inbuf->linebuf
;-----------------------------------------------------------------------
lce_comp:	rcall	lce_setptr		;set pointers

lce_comp01:	rcall	lce_getchar		;get char
		sts	bas_ram+8,YL		;save RAM pointer
		sts	bas_ram+9,YH
		mov	r10,tempreg1		;save last char to r10

;check for comment
lce_comment:	cpi	tempreg1,0x27		;start of comment
		brne	lce_string
lce_comment1:	st	X+,tempreg1		;store char
		rcall	lce_getchar		;get next char
		rjmp	lce_comment1		;loop
		
;check for string
lce_string:	cpi	tempreg1,'"'		;end of string
		brne	lce_keyw		
lce_string1:	st	X+,tempreg1		;store char
		rcall	lce_getchar		;get next char
		cpi	tempreg1,'"'
		brne	lce_string1
lce_putchar:	st	X+,tempreg1		;store char
		rjmp	lce_comp01		;loop

;check for keyword
lce_keyw:    	cpi	tempreg1,'a'		;a-char
		brcs	lce_upc1		;less
		cpi	tempreg1,'z'+1		;z-char
		brcc	lce_nokeyw		;greater
		subi	tempreg1,0x20		;-32
	
lce_upc1:    	mov	r10,tempreg1
		cpi	tempreg1,'A'		;A-char
		brcs	lce_nokeyw		;no keyword
		cpi	tempreg1,'Z'+1		;Z-char
		brcc	lce_nokeyw		;no keyword
		ld	tempreg2,Y+		;get next char
	 	cpi	tempreg2,'a'		;a-char
		brcs	lce_upc2		;less
		cpi	tempreg2,'z'+1		;z-char
		brcc	lce_nokeyw		;greater
		subi	tempreg2,0x20		;-32
	
lce_upc2:    	cpi	tempreg2,'A'		;A-char
		brcs	lce_nokeyw		;no keyword
		cpi	tempreg2,'Z'+1		;Z-char
		brcc	lce_nokeyw		;no keyword
		
		ldi	tempreg4,0xc0		;first (abbreviated) token
		ldi	ZL,LOW(lce_toktab*2)	;Start of keyword table
		ldi	ZH,HIGH(lce_toktab*2)

lce_keyw1:	lpm	tempreg3,Z+		;get first ROM char
		cpi	tempreg3,0x00		;end of table
		breq	lce_nokeyw		;no keyword 
		
		cp	tempreg1,tempreg3	;compare first char
		brne	lce_keyw2
		lpm	tempreg3,Z+
		cp	tempreg2,tempreg3	;compare second char
		brne	lce_keyw2
		rjmp	lce_keyw3		;begin of keyword found

lce_keyw2:	lpm	tempreg3,Z+		;get ROM char
		cpi	tempreg3,32		;wait for space in rom table
		brne	lce_keyw2		;loop
		inc	tempreg4		;token+1
		rjmp	lce_keyw1		;next check

lce_nokeyw:	mov	tempreg1,r10
		lds	YH,bas_ram+9
		lds	YL,bas_ram+8
		rjmp	lce_putchar

;it might be a keyword
				
lce_keyw3:	ld	tempreg1,Y+		;get next RAM char
    		cpi	tempreg1,'a'		;a-char
		brcs	lce_upc3		;less
		cpi	tempreg1,'z'+1		;z-char
		brcc	lce_upc3		;greater
		subi	tempreg1,0x20		;-32
	
lce_upc3:	lpm	tempreg3,Z+		;get next ROM char
		cpi	tempreg3,32		;end of keyword
		brne	lce_keyw4		;no
		cpi	tempreg1,'('		;its a function?
		breq	lce_nokeyw		;ignore function keywords
		andi	tempreg4,0xbf		;clear abbrev flag
		rjmp	lce_wtoken		;write token

lce_keyw4:	cpi	tempreg1,'('		;its a function?
		breq	lce_nokeyw		;ignore function keywords
		cpi	tempreg1,' '		;space?
		breq	lce_wtoken		;yes
		cpi	tempreg1,':'		;separator
		breq	lce_wtoken		;yes
		cpi	tempreg1,0x00		;EOL
		breq	lce_wtoken		;yes
		cp	tempreg1,tempreg3	;RAM=ROM?
		brne	lce_nokeyw			
		rjmp	lce_keyw3

lce_wtoken:	sbiw	YL,1						
lce_wtoken0:	ld	tempreg1,Y+		;get char
		cpi	tempreg1,0x20		;space
		breq	lce_wtoken0
lce_wtoken1:	sbiw	YL,1						
		mov	tempreg1,tempreg4
		rjmp	lce_putchar

;-----------------------------------------------------------------------
; get char from inbuf and check for line end
;-----------------------------------------------------------------------
lce_getchar:	ld	tempreg1,Y+		;get char from inbuf
		cpi	tempreg1,0x00		;EOL
		breq	lce_getchar1		;branch if yes
		ret				;normal return
lce_getchar1:	ldi	r17,0xff		;EOL-code
		st	X+,r17			;write to linebuf
    		pop	tempreg1		;kill last call from stack
		pop	tempreg1	
		ret
		
;-----------------------------------------------------------------------
; set pointers
;-----------------------------------------------------------------------
lce_setptr:	ldi	YL,LOW(bas_inbuf+3)	;text buffer address
		ldi	YH,HIGH(bas_inbuf+3)
		ldi	XL,LOW(bas_linebuf)	;code buffer address
		ldi	XH,HIGH(bas_linebuf)
		ret

;-----------------------------------------------------------------------
; expand linebuf -> inbuf
;-----------------------------------------------------------------------
lce_xpand:	rcall	lce_setptr
		rjmp	lce_xpand_000
lce_xpand_00:	st	Y+,tempreg4		;write char
lce_xpand_000:	ld	tempreg4,X+		;get char
		cpi	tempreg4,0xff		;EOL
		brne	lce_xpand_01		;no
		st	Y,byte0			;write EOL
		ret				;end
		
lce_xpand_01:	sbrs	tempreg4,7		;skip if token
		rjmp	lce_xpand_00		;char
		
		ldi	ZL,LOW(lce_toktab*2)	;Start of keyword table
		ldi	ZH,HIGH(lce_toktab*2)
		mov	tempreg3,tempreg4	;save token
		andi	tempreg4,0x3f		;64 token

lce_xpand_02:	cpi	tempreg4,0x00
		breq	lce_xpand_04
lce_xpand_03:	lpm	tempreg2,Z+		;get char
		cpi	tempreg2,0x20		;space		
		brne	lce_xpand_03
		dec	tempreg4
		brne	lce_xpand_02

lce_xpand_04:	lpm	tempreg2,Z+		;get ROM char 1
		st	Y+,tempreg2		;write	
		lpm	tempreg2,Z+		;get ROM char 2
		st	Y+,tempreg2		;write	
		ldi	tempreg2,0x20		;space
		sbrs	tempreg3,6		;abbrev?
lce_xpand_05:	lpm	tempreg2,Z+
		st	Y+,tempreg2
		cpi	tempreg2,0x20		;end of keyword
		brne	lce_xpand_05
		rjmp	lce_xpand_000		

		
;------------------------------------------------------------------------------
; write line [ctrl] from inbuf to vram
;------------------------------------------------------------------------------
lce_wline:	rcall	lce_calc		;set address in vram
		ldi	YL,LOW(bas_inbuf)	;text buffer address
		ldi	YH,HIGH(bas_inbuf)
		ldi	tempreg2,libmio_ecols	;number of chars to copy
lce_wline1:	ld	tempreg1,Y+		;get from vram
		st	X+,tempreg1
		dec	tempreg2
		brne	lce_wline1		
		ret

;------------------------------------------------------------------------------
; read line [ctrl] from vram to inbuf
;------------------------------------------------------------------------------
lce_rline:	rcall	lce_calc		;set address in vram
		ldi	YL,LOW(bas_inbuf)	;text buffer address
		ldi	YH,HIGH(bas_inbuf)
		ldi	tempreg2,libmio_ecols	;number of chars to copy
lce_rline1:	ld	tempreg1,X+		;get from vram
		st	Y+,tempreg1
		dec	tempreg2
		brne	lce_rline1		
		st	Y+,byte0		;add an extra EOL
		ret

;------------------------------------------------------------------------------
; clear inbuf and write line number of [ctrl]
;------------------------------------------------------------------------------
lce_clear:	ldi	ZL,LOW(bas_inbuf)	;text buffer address
		ldi	ZH,HIGH(bas_inbuf)
		ldi	tempreg4,0x20		;space
		ldi	tempreg3,libmio_ecols
lce_clear1:	st	Z+,tempreg4
		dec	tempreg3
		brne	lce_clear1	
		
lce_number:	ldi	ZL,LOW(bas_inbuf)	;text buffer address
		ldi	ZH,HIGH(bas_inbuf)
		ldi	tempreg4,0x30		;"0"
		mov	tempreg3,ctrl
		inc	tempreg3		;line numbers start with 1
lce_number1:	subi	tempreg3,10		;-10
		brcs	lce_number2
		inc	tempreg4
		rjmp	lce_number1
lce_number2:	subi	tempreg3,0xc6		;+70
		st	Z+,tempreg4		;10^1
		st	Z+,tempreg3		;10^0
		ldi	tempreg4,0x20		;space
		st	Z+,tempreg4		;write
		ret

;------------------------------------------------------------------------------
; calc line adr of ctrl to X
;------------------------------------------------------------------------------
lce_calc:	ldi	XH,libmio_ecols		;bytes/vram-line
		mul	XH,ctrl
	    	ldi	XL,LOW(libmio_vram+3*libmio_cols)	;lines start
		ldi	XH,HIGH(libmio_vram+3*libmio_cols)
		add	XL,r0			;calc line address
		adc	XH,r1
		ret

;------------------------------------------------------------------------------
; shrink inbuf
;------------------------------------------------------------------------------
lce_shrink:	ldi	YL,LOW(bas_inbuf+libmio_ecols)
		ldi	YH,HIGH(bas_inbuf+libmio_ecols)
		ldi	tempreg2,libmio_ecols-3	;max chars
lce_shrink1:	ld	tempreg1,-Y		;		
		cpi	tempreg1,0x20		;space
		brne	lce_shrink2
		st	Y,byte0			;write EOL	
		dec	tempreg2
		brne	lce_shrink1
lce_shrink2:	ret		
			
;------------------------------------------------------------------------------
; load program from Flash
;------------------------------------------------------------------------------
lce_pload:	clr	ctrl			;first line is 0
lce_pload1:	call	mem_readline		;get from EEPROM
		rcall	lce_clear		;clear inbuf
		rcall	lce_xpand		;expand line
		rcall	lce_wline		;write into vram
		inc	ctrl			;next line
		cpi	ctrl,bas_lines		;all lines done
		brne	lce_pload1		;no
		ret

;------------------------------------------------------------------------------
; save program to Flash (editor mode)
;------------------------------------------------------------------------------
lce_psave:	ldi	r16,0x00		;black
		ldi	r17,0x02		;short int
		libmio_sync
		libmio_sync
		clr	ctrl			;first line is 0
lce_psave1:	rcall	lce_rline		;get line from vram
		rcall	lce_shrink		;set EOL
		rcall	lce_comp		;compress line
		call	mem_writeline		;write to flash
		inc	ctrl			;next line
		cpi	ctrl,bas_lines		;all lines done
		brne	lce_psave1		;no
		ldi	YL,LOW(libmio_vram+11)	;adr of filename
		ldi	YH,HIGH(libmio_vram+11)
		rjmp	lce_rpsave3

;------------------------------------------------------------------------------
; save program to Flash (receive mode)
;------------------------------------------------------------------------------
lce_rpsave:	ldi	r16,0x00		;black
		ldi	r17,0x02		;short int
		clr	ctrl
lce_rpsave1:	rcall	lce_rline		;get line from vram
		rcall	lce_shrink		;set EOL
		rcall	lce_comp		;compress line
		call	mem_writeline		;write to flash
		inc	ctrl			;next line
		cpi	ctrl,bas_lines		;all lines done
		brne	lce_rpsave1		;no
		ldi	YL,LOW(libmio_vram+libmio_cols)	;adr of fname
		ldi	YH,HIGH(libmio_vram+libmio_cols)
lce_rpsave2:	ld	tempreg1,Y+
		cpi	tempreg1,0x20		;space
		brne	lce_rpsave2		
lce_rpsave3:	call	mem_writename		;write to inbuf
		call	mem_writeline		;write to flash
		ret

;------------------------------------------------------------------------------
; find last not empty line
;------------------------------------------------------------------------------
lce_findlast:	lds	ZL,libmio_ram+45	;startadr program
		lds	ZH,libmio_ram+46
		ldi	YL,LOW(1568)		;offset to line 50
		ldi	YH,HIGH(1568)
		add	ZL,YL
		adc	ZH,YH
		ldi	r19,bas_lines-1		;lines=50
lce_findlast1:	lpm	XL,Z			;get first char
		cpi	XL,0xff			;EOL
		brne	lce_findlast2
		sbiw	ZL,32			;next line
		dec	r19
		brne	lce_findlast1
lce_findlast2:	inc	r19
		
;------------------------------------------------------------------------------
; list program
;------------------------------------------------------------------------------
lce_plist1:	clr	ctrl			;first line is 0
lce_plist2:	call	mem_readline		;get from EEPROM
		rcall	lce_clear		;clear inbuf
		rcall	lce_xpand		;expand line
		rcall	lce_shrink		;set EOL
		push	ctrl
		ldi	ZL,LOW(bas_inbuf)	;set pointer
		ldi	ZH,HIGH(bas_inbuf)
		libmio_ramtext			;output text
		libmio_newline
		pop	ctrl
		inc	ctrl			;next line
		cp	ctrl,r19		;all lines done
		brne	lce_plist2		;no
		ret

;------------------------------------------------------------------------------
; clear editor lines (new program)
;------------------------------------------------------------------------------
lce_pnew:	clr	ctrl			;first line is 0
		lds	tempreg1,libmio_ram+50
		call	mem_setprog
lce_pnew1:	rcall	lce_clear		;clear inbuf
		rcall	lce_wline		;write into vram
		inc	ctrl
		cpi	ctrl,bas_lines		;all lines done
		brne	lce_pnew1		;no
		ret

;------------------------------------------------------------------------------
; renumber lines
;------------------------------------------------------------------------------
lce_renumber:	clr	ctrl			;first line is 0
		lds	tempreg1,libmio_ram+50
		call	mem_setprog
lce_renumber1:	rcall	lce_rline		;read from vram
		rcall	lce_number		;set correct number
		rcall	lce_wline		;write into vram
		inc	ctrl
		cpi	ctrl,bas_lines		;all lines done
		brne	lce_renumber1		;no
		ret

;------------------------------------------------------------------------------
; delete line [ctrl] in vram
;------------------------------------------------------------------------------
lce_del:	cpi	ctrl,bas_lines-1	;last line
		brcs	lce_del0
		ret
lce_del0:	rcall	lce_calc		;get line addr to X
		mov	YL,XL
		mov	YH,XH
lce_del1:	ldi	tempreg2,libmio_ecols	;chars per line
lce_del2:	ldd	tempreg1,Y+libmio_ecols	;char of next line
		st	Y+,tempreg1		;write to recent line
		dec	tempreg2
		brne	lce_del2		;char loop
		inc	ctrl			;next line
		cpi	ctrl,bas_lines-1
		brne	lce_del1		;line loop
		rcall	lce_clear		;clear buffer line
		rcall	lce_wline
		rjmp	lce_renumber		;renumber lines 
		
;------------------------------------------------------------------------------
; insert line [ctrl] in vram
;------------------------------------------------------------------------------
lce_ins:	cpi	ctrl,bas_lines-1	;last line
		brcs	lce_ins0
		ret
lce_ins0:	mov	tempreg3,ctrl		;copy 
		ldi	ctrl,bas_lines-1		
		rcall	lce_calc		;get line of line max-1
		mov	YL,XL
		mov	YH,XH
lce_ins1:	ldi	tempreg2,libmio_ecols	;chars per line
lce_ins2:	ld	tempreg1,-Y		;char of this line
		std	Y+libmio_ecols,tempreg1
		dec	tempreg2
		brne	lce_ins2		;char loop
		dec	ctrl			;next line
		cp	ctrl,tempreg3
		brne	lce_ins1		;line loop
		rcall	lce_calc		;get line of line max-1
		rcall	lce_clear		;clear last line
		rcall	lce_wline
		rjmp	lce_renumber		;renumber lines 

;------------------------------------------------------------------------------
; token table
;------------------------------------------------------------------------------
lce_toktab:	.db "PRINT CLS GOTO BOX FBOX SYNC END IF THEN POS NOTE EMIT"
		.db " WKEY RKEY CALL RETURN FOR NEXT INPUT PLOT WAIT TSET"
		.db " TGET ESET EGET SPUT SGET EPOKE DIR OUT COLOR LCHAR "
		.db "LPIX PUMP XPOKE YEMIT LIMIT GCHAR DRAW SDEF SSHOW SHIDE "
		.db "BREAK FAST SLOW XCALL ICOMM DATA CTEXT FREAD FWRITE "
		.db "XX XX XX XX XX XX XX XX XX XX XX XX XX ",0
								